package com.tzld.longarticle.recommend.server.service.recommend;

import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.google.common.collect.Lists;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.tzld.longarticle.recommend.server.common.CommonThreadPoolExecutor;
import com.tzld.longarticle.recommend.server.common.enums.StatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.aigc.PublishContentTypeEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.ArticleCategoryStatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.ArticlePoolPromotionSourceStatusEnum;
import com.tzld.longarticle.recommend.server.mapper.aigc.AigcBaseMapper;
import com.tzld.longarticle.recommend.server.mapper.crawler.CrawlerBaseMapper;
import com.tzld.longarticle.recommend.server.mapper.longArticle.LongArticleBaseMapper;
import com.tzld.longarticle.recommend.server.model.dto.CrawlerContent;
import com.tzld.longarticle.recommend.server.model.dto.kimi.KimiResult;
import com.tzld.longarticle.recommend.server.model.entity.aigc.ProducePlanExeRecord;
import com.tzld.longarticle.recommend.server.model.entity.aigc.PublishAccount;
import com.tzld.longarticle.recommend.server.model.entity.aigc.PublishContent;
import com.tzld.longarticle.recommend.server.model.entity.aigc.PublishContentOutput;
import com.tzld.longarticle.recommend.server.model.entity.crawler.Article;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.ArticleCategory;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.ArticleCrawlerPlan;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.ArticlePoolPromotionSource;
import com.tzld.longarticle.recommend.server.model.param.ArticleFindSourceParam;
import com.tzld.longarticle.recommend.server.model.vo.ProduceContentCrawlerVO;
import com.tzld.longarticle.recommend.server.model.vo.RootPublishContentVO;
import com.tzld.longarticle.recommend.server.remote.CrawlerContentByPlanService;
import com.tzld.longarticle.recommend.server.remote.KimiApiService;
import com.tzld.longarticle.recommend.server.repository.aigc.PublishAccountRepository;
import com.tzld.longarticle.recommend.server.repository.aigc.PublishContentOutputRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.AccountAvgInfoRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.ArticleRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.ArticleCategoryRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.ArticleCrawlerPlanRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.ArticlePoolPromotionSourceRepository;
import com.tzld.longarticle.recommend.server.service.recommend.config.AccountIndexAvgViewCountService;
import com.tzld.longarticle.recommend.server.util.DateUtils;
import com.tzld.longarticle.recommend.server.util.Md5Util;
import com.tzld.longarticle.recommend.server.util.TitleSimilarCheckUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.concurrent.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author dyp
 */
@Service
@Slf4j
public class ArticleService {

    @Autowired
    AccountIndexAvgViewCountService accountIndexAvgViewCountService;
    @Autowired
    AccountAvgInfoRepository accountAvgInfoRepository;
    @Autowired
    ArticleRepository articleRepository;
    @Autowired
    PublishContentOutputRepository publishContentOutputRepository;
    @Autowired
    PublishAccountRepository publishAccountRepository;
    @Autowired
    AigcBaseMapper aigcBaseMapper;
    @Autowired
    CrawlerBaseMapper crawlerBaseMapper;
    @Autowired
    LongArticleBaseMapper longArticleBaseMapper;
    @Autowired
    ArticlePoolPromotionSourceRepository articlePoolPromotionSourceRepository;
    @Autowired
    ArticleCrawlerPlanRepository articleCrawlerPlanRepository;
    @Autowired
    ArticleCategoryRepository articleCategoryRepository;
    @Autowired
    CrawlerContentByPlanService crawlerContentByPlanService;
    @Autowired
    KimiApiService kimiApiService;

    @ApolloJsonValue("${cold.pool.produce.planId:[\"20240802021606053813696\", \"20240802080355355308981\",\n" +
            "\"20240805154433785506170\", \"20240805154359027876170\", \"20241024100016206421084\", " +
            "\"20241030070010871546586\"]}")
    private static List<String> producePlanIds;
    @Value("${kimiCategoryPrompt:}")
    private String kimiCategoryPrompt;

    private final static ExecutorService pool = new CommonThreadPoolExecutor(
            32,
            128,
            0L, TimeUnit.SECONDS,
            new LinkedBlockingQueue<>(1000),
            new ThreadFactoryBuilder().setNameFormat("DEFAULT-%d").build(),
            new ThreadPoolExecutor.AbortPolicy());

    public void findSource(ArticleFindSourceParam param) {
        if (StringUtils.hasText(param.getWxSn())) {
            syncAigcIdByWxSn(param.getWxSn());
            return;
        }
        long minTimestamp;
        if (StringUtils.hasText(param.getDateStr())) {
            minTimestamp = DateUtils.dateStrToTimestamp(param.getDateStr(), "yyyyMMdd") - 86400 * 7;
        } else {
            minTimestamp = DateUtils.getTodayStart() - 86400 * 7;
        }
        while (true) {
            List<Article> articleList = crawlerBaseMapper.getWaitingFindArticle(minTimestamp);
            if (CollectionUtils.isEmpty(articleList)) {
                return;
            }
            CountDownLatch cdl = new CountDownLatch(articleList.size());
            for (Article article : articleList) {
                pool.execute(() -> {
                    try {
                        syncAigcIdByWxSn(article.getWxSn());
                    } finally {
                        cdl.countDown();
                    }
                });
                minTimestamp = minTimestamp > article.getPublishTimestamp() ? minTimestamp : article.getPublishTimestamp();
            }
            try {
                cdl.await();
            } catch (Exception e) {
                log.error("cdl error", e);
            }
            log.info("findSource timestamp:{}", minTimestamp);
        }
    }

    private void syncAigcIdByWxSn(String wxSn) {
        Article article = articleRepository.getByWxSn(wxSn);
        String ghId = article.getGhId();
        String title = article.getTitle();
        Long publishTimestamp = article.getPublishTimestamp() * 1000;
        PublishAccount publishAccount = publishAccountRepository.getByGhId(ghId);
        List<PublishContent> publishContentList = aigcBaseMapper.getNearestPublishContent(publishAccount.getId(), publishTimestamp, 100);
        if (CollectionUtils.isEmpty(publishContentList)) {
            return;
        }
        String publishContentId = null;
        String channelContentId = null;

        PublishContent publishContent = findPublishContent(publishContentList, title, publishTimestamp);
        if (Objects.nonNull(publishContent)) {
            publishContentId = publishContent.getId();
            channelContentId = publishContent.getCrawlerChannelContentId();
        } else {
            publishContentList = aigcBaseMapper.getLateNearestPublishContent(publishAccount.getId(), publishTimestamp);
            publishContent = findPublishContent(publishContentList, article.getTitle(), publishTimestamp);
            if (Objects.nonNull(publishContent)) {
                publishContentId = publishContent.getId();
                channelContentId = publishContent.getCrawlerChannelContentId();
            }
        }
        log.info("syncAigcIdByWxSn titleMatch finish");
        if (Objects.isNull(channelContentId)) {
            return;
        }
        // 更新 official_article_v2
        crawlerBaseMapper.updateArticleAigcId(wxSn, publishContentId, channelContentId);
        // 查找记录根记录
        long start = System.currentTimeMillis();
        RootPublishContentVO result = getRootPublishContent(channelContentId, null, publishContentId, null, 0);
        log.info("syncAigcIdByWxSn getRootPublishContent finish cost:{}", System.currentTimeMillis() - start);
        // 更新source root publish_content_id
        crawlerBaseMapper.updateArticleSourceRootId(wxSn, result.getSourcePublishContentId(), result.getRootPublishContentId(),
                result.getRootProduceContentId());
    }

    private PublishContent findPublishContent(List<PublishContent> publishContentList,
                                              String title,
                                              Long publishTimestamp) {
        if (CollectionUtils.isEmpty(publishContentList)) {
            return null;
        }
        Map<String, PublishContent> publishContentMap = publishContentList.stream().collect(
                Collectors.toMap(PublishContent::getId, publishContent -> publishContent));
        List<String> publishContentIds = publishContentList.stream().map(PublishContent::getId).collect(Collectors.toList());
        List<PublishContentOutput> publishContentOutputList = publishContentOutputRepository.
                getByPublishContentIdInAndContentTypeAndSelectStatus(publishContentIds, PublishContentTypeEnum.title.getVal(), 1);
        Map<String, List<PublishContentOutput>> publishContentOutputMap = publishContentOutputList.stream().collect(
                Collectors.groupingBy(PublishContentOutput::getOutput));
        List<String> titles = publishContentOutputList.stream().map(PublishContentOutput::getOutput).collect(Collectors.toList());
        PublishContent publishContent = null;
        if (titles.contains(title)) {
            publishContent = getPublishContentByTitle(publishContentOutputMap, publishContentMap, title,
                    publishTimestamp);
        } else {
            for (String aTitle : titles) {
                if (TitleSimilarCheckUtil.isSimilar(title, aTitle, TitleSimilarCheckUtil.SIMILARITY_THRESHOLD)) {
                    publishContent = getPublishContentByTitle(publishContentOutputMap, publishContentMap, aTitle,
                            publishTimestamp);
                    break;
                }
            }
        }
        return publishContent;
    }

    private PublishContent getPublishContentByTitle(Map<String, List<PublishContentOutput>> publishContentOutputMap,
                                                    Map<String, PublishContent> publishContentMap,
                                                    String title,
                                                    Long publishTimestamp) {
        List<PublishContentOutput> outputList = publishContentOutputMap.get(title);
        List<PublishContent> publishContents = outputList.stream().map(o -> publishContentMap.get(o.getPublishContentId()))
                .collect(Collectors.toList());
        return getNearestContent(publishContents, publishTimestamp);
    }

    private PublishContent getNearestContent(List<PublishContent> publishContents, Long publishTimestamp) {
        if (publishContents.size() == 1) {
            return publishContents.get(0);
        }
        PublishContent result = null;
        Long nearest = 0L;
        for (PublishContent publishContent : publishContents) {
            Long timestamp = publishContent.getPublishTimestamp();
            if (Objects.isNull(result)) {
                result = publishContent;
                nearest = timestamp;
                continue;
            }
            if (Math.abs(timestamp - publishTimestamp) < Math.abs(nearest - publishTimestamp)) {
                result = publishContent;
                nearest = timestamp;
            }
        }
        return result;
    }

    public RootPublishContentVO getRootPublishContent(String channelContentId,
                                                      String sourcePublishContentId,
                                                      String rootPublishContentId,
                                                      String rootProduceContentId,
                                                      int times) {
        RootPublishContentVO result = new RootPublishContentVO();
        result.setChannelContentId(channelContentId);
        result.setSourcePublishContentId(sourcePublishContentId);
        result.setRootPublishContentId(rootPublishContentId);
        result.setRootProduceContentId(rootProduceContentId);
        if (times > 20) {
            return result;
        }
        CrawlerContent crawlerContent = aigcBaseMapper.getCrawlerContentByChannelContentId(channelContentId);
        if (Objects.isNull(crawlerContent) || !StringUtils.hasText(crawlerContent.getGhId())) {
            return result;
        }
        PublishAccount publishAccount = publishAccountRepository.getByGhId(crawlerContent.getGhId());
        if (Objects.isNull(publishAccount)) {
            return result;
        }
        String title = crawlerContent.getTitle();
        Long publishTimestamp = crawlerContent.getPublishTimestamp();
        List<PublishContent> publishContentList = aigcBaseMapper.getNearestPublishContent(publishAccount.getId(), publishTimestamp, 100);
        if (CollectionUtils.isEmpty(publishContentList)) {
            return result;
        }
        PublishContent publishContent = findPublishContent(publishContentList, title, publishTimestamp);
        if (Objects.nonNull(publishContent)) {
            if (!StringUtils.hasText(sourcePublishContentId)) {
                result.setSourcePublishContentId(publishContent.getId());
            }
            result.setRootPublishContentId(publishContent.getId());
            result.setRootProduceContentId(publishContent.getSourceId());
            channelContentId = publishContent.getCrawlerChannelContentId();
        } else {
            publishContentList = aigcBaseMapper.getLateNearestPublishContent(publishAccount.getId(), publishTimestamp);
            publishContent = findPublishContent(publishContentList, title, publishTimestamp);
            if (Objects.nonNull(publishContent)) {
                if (!StringUtils.hasText(sourcePublishContentId)) {
                    result.setSourcePublishContentId(publishContent.getId());
                }
                result.setRootPublishContentId(publishContent.getId());
                result.setRootProduceContentId(publishContent.getSourceId());
                channelContentId = publishContent.getCrawlerChannelContentId();
            }
        }
        // channelContentId未被修改，说明未找到
        if (channelContentId.equals(crawlerContent.getChannelContentId())) {
            return result;
        } else {
            result = getRootPublishContent(channelContentId, result.getSourcePublishContentId(),
                    result.getRootPublishContentId(), result.getRootProduceContentId(), ++times);
            if (!channelContentId.equals(result.getChannelContentId())) {
                result.getMidChannelContentIds().add(channelContentId);
            }
            return result;
        }
    }

    public void articlePromotionTraceability(String channelContentId) {
        // 查找待处理任务
        List<ArticlePoolPromotionSource> tasks = new ArrayList<>();
        if (StringUtils.hasText(channelContentId)) {
            ArticlePoolPromotionSource item = articlePoolPromotionSourceRepository.getByChannelContentId(channelContentId);
            tasks.add(item);
        } else {
            tasks = articlePoolPromotionSourceRepository.getByStatusAndDeletedAndWxSnIsNotNull(0, 0);
        }
        long now = System.currentTimeMillis();
        for (ArticlePoolPromotionSource task : tasks) {
            // 判断文章是否被抓回来，如果没有抓回来，不进行处理 临时方案
            CrawlerContent crawlerContent = aigcBaseMapper.getCrawlerContentByChannelContentId(task.getChannelContentId());
            if (Objects.isNull(crawlerContent)) {
                continue;
            }
            // 溯源
            Article article = articleRepository.getByWxSn(task.getWxSn());
            if (Objects.isNull(article)) {
                task.setDeleted(StatusEnum.ONE.getCode());
                articlePoolPromotionSourceRepository.save(task);
                continue;
            }
            PublishAccount publishAccount = publishAccountRepository.getByGhId(article.getGhId());
            if (Objects.isNull(publishAccount)) {
                continue;
            }
            long publishTimestamp = article.getPublishTimestamp() > 0 ? article.getPublishTimestamp() * 1000 : article.getUpdateTime() * 1000;
            List<PublishContent> publishContentList = aigcBaseMapper.getNearestPublishContent(publishAccount.getId(), publishTimestamp, 100);
            PublishContent publishContent = findPublishContent(publishContentList, task.getTitle(), publishTimestamp);
            if (Objects.isNull(publishContent)) {
                publishContentList = aigcBaseMapper.getLateNearestPublishContent(publishAccount.getId(), publishTimestamp);
                publishContent = findPublishContent(publishContentList, task.getTitle(), publishTimestamp);
                if (Objects.isNull(publishContent)) {
                    task.setDeleted(StatusEnum.ONE.getCode());
                    articlePoolPromotionSourceRepository.save(task);
                    continue;
                }
            }
            RootPublishContentVO source = getRootPublishContent(publishContent.getCrawlerChannelContentId(), null, publishContent.getId(), publishContent.getSourceId(), 0);
            // 更新
            if (StringUtils.hasText(source.getRootProduceContentId())) {
                task.setStatus(ArticlePoolPromotionSourceStatusEnum.FINISH.getCode());
                task.setSourcePublishContentId(source.getSourcePublishContentId());
                task.setRootProduceContentId(source.getRootProduceContentId());
                task.setRootPublishContentId(source.getRootPublishContentId());
                task.setUpdateTimestamp(now);
                articlePoolPromotionSourceRepository.save(task);
                // 保存中间环节 晋级溯源
                for (String midChannelContentId : source.getMidChannelContentIds()) {
                    ArticlePoolPromotionSource item = new ArticlePoolPromotionSource();
                    BeanUtils.copyProperties(task, item);
                    ArticlePoolPromotionSource dto = articlePoolPromotionSourceRepository.getByChannelContentId(midChannelContentId);
                    if (Objects.nonNull(dto)) {
                        // 以dto为基础
                        dto.setRootProduceContentId(task.getRootProduceContentId());
                        dto.setRootPublishContentId(task.getRootPublishContentId());
                        dto.setStatus(task.getStatus());
                        dto.setLevel(task.getLevel());
                        dto.setUpdateTimestamp(task.getUpdateTimestamp());
                        item = dto;
                    } else {
                        // 以新item为基础
                        item.setChannelContentId(midChannelContentId);
                        item.setCreateTimestamp(item.getUpdateTimestamp());
                    }
                    item.setDeleted(StatusEnum.ZERO.getCode());
                    articlePoolPromotionSourceRepository.save(item);
                }
                longArticleBaseMapper.updateRootProduceContentLevel(task.getRootProduceContentId(), task.getLevel());
            } else {
                task.setDeleted(StatusEnum.ONE.getCode());
                articlePoolPromotionSourceRepository.save(task);
            }
        }
    }


    public void articleCategory() {
        addArticleCategoryByCrawlerPlan();
        addArticleCategoryByProducePlan();
        dealArticleCategory();
    }

    private void dealArticleCategory() {
        List<ArticleCategory> dealList = articleCategoryRepository.getByStatus(ArticleCategoryStatusEnum.WAITING.getCode());
        List<List<ArticleCategory>> partitionList = Lists.partition(dealList, 20);
        for (List<ArticleCategory> partition : partitionList) {
            List<String> partitionTitles = partition.stream().map(ArticleCategory::getTitle).distinct().collect(Collectors.toList());
            String prompt = buildKimiPrompt(partitionTitles);
            KimiResult kimiResult = kimiApiService.requestOfficialApi(prompt, null, null);
            long now = System.currentTimeMillis();
            JSONObject obj = null;
            if (kimiResult.isSuccess()) {
                try {
                    obj = JSONObject.parseObject(kimiResult.getResponse().getChoices().get(0).getMessage().getContent());
                } catch (Exception e) {
                    log.error(kimiResult.getResponse().getChoices().get(0).getMessage().getContent());
                }
            }
            for (ArticleCategory articleCategory : partition) {
                articleCategory.setKimiResult(kimiResult.getResponseStr());
                articleCategory.setUpdateTimestamp(now);
                if (kimiResult.isSuccess() && Objects.nonNull(obj) && obj.containsKey(articleCategory.getTitle())) {
                    articleCategory.setCategory(obj.getString(articleCategory.getTitle()));
                    articleCategory.setStatus(ArticleCategoryStatusEnum.SUCCESS.getCode());
                } else {
                    articleCategory.setStatus(ArticleCategoryStatusEnum.FAIL.getCode());
                    articleCategory.setFailReason(kimiResult.getFailReason());
                }
                articleCategoryRepository.save(articleCategory);
            }
        }
    }

    private void addArticleCategoryByCrawlerPlan() {
        List<ArticleCrawlerPlan> articleCrawlerPlanList = articleCrawlerPlanRepository.getByStatus(StatusEnum.ZERO.getCode());
        for (ArticleCrawlerPlan crawlerPlan : articleCrawlerPlanList) {
            List<ProduceContentCrawlerVO> list = crawlerContentByPlanService.getCrawlerContentByPlan(crawlerPlan.getCrawlerPlanId(), producePlanIds);
            List<String> produceContentIds = list.stream().map(ProduceContentCrawlerVO::getProduceContentId).collect(Collectors.toList());
            List<ArticleCategory> exists = articleCategoryRepository.getByProduceContentIdIn(produceContentIds);
            List<String> existsIds = exists.stream().map(ArticleCategory::getProduceContentId).collect(Collectors.toList());
            list = list.stream().filter(o -> !existsIds.contains(o.getProduceContentId())).collect(Collectors.toList());
            long now = System.currentTimeMillis();
            List<ArticleCategory> saveList = new ArrayList<>();
            for (ProduceContentCrawlerVO vo : list) {
                ArticleCategory item = new ArticleCategory();
                item.setCrawlerPlanId(crawlerPlan.getCrawlerPlanId());
                item.setChannelContentId(vo.getChannelContentId());
                item.setProduceContentId(vo.getProduceContentId());
                item.setTitle(vo.getTitle());
                item.setTitleMd5(Md5Util.encoderByMd5(vo.getTitle()));
                item.setCreateTimestamp(now);
                saveList.add(item);
            }
            if (CollectionUtils.isNotEmpty(saveList)) {
                longArticleBaseMapper.batchInsertArticleCategory(saveList);
            }
            // 抓取计划超过5天设置为已处理
            String dateStr = crawlerPlan.getCrawlerPlanId().substring(0, 8);
            if (DateUtils.dateStrToTimestamp(dateStr, "yyyyMMdd") < now - 86400 * 5) {
                crawlerPlan.setStatus(StatusEnum.ONE.getCode());
                crawlerPlan.setUpdateTimestamp(now);
                articleCrawlerPlanRepository.save(crawlerPlan);
            }
        }
    }

    private void addArticleCategoryByProducePlan() {
        List<ProducePlanExeRecord> produceContentList = aigcBaseMapper.getAllByProducePlanId(producePlanIds);
        List<String> channelContentIds = produceContentList.stream().map(ProducePlanExeRecord::getChannelContentId).distinct().collect(Collectors.toList());
        List<ArticleCategory> articleCategoryList = articleCategoryRepository.getAllByChannelContentIdIn(channelContentIds);
        List<String> articleCategoryIds = articleCategoryList.stream().map(ArticleCategory::getChannelContentId).collect(Collectors.toList());
        List<ProduceContentCrawlerVO> list = produceContentList.stream().filter(o -> !articleCategoryIds.contains(o.getChannelContentId())).map(o -> {
            ProduceContentCrawlerVO item = new ProduceContentCrawlerVO();
            item.setChannelContentId(o.getChannelContentId());
            item.setProduceContentId(o.getPlanExeId());
            return item;
        }).collect(Collectors.toList());
        channelContentIds = channelContentIds.stream().filter(o -> !articleCategoryIds.contains(o)).collect(Collectors.toList());
        List<CrawlerContent> crawlerContentList = aigcBaseMapper.getCrawlerContentByChannelContentIdIn(channelContentIds);
        Map<String, CrawlerContent> map = crawlerContentList.stream().collect(Collectors.toMap(CrawlerContent::getChannelContentId, Function.identity()));
        long now = System.currentTimeMillis();
        List<ArticleCategory> saveList = new ArrayList<>();
        for (ProduceContentCrawlerVO vo : list) {
            ArticleCategory item = new ArticleCategory();
            item.setChannelContentId(vo.getChannelContentId());
            item.setProduceContentId(vo.getProduceContentId());
            CrawlerContent crawlerContent = map.get(vo.getChannelContentId());
            if (Objects.nonNull(crawlerContent)) {
                String title = crawlerContent.getTitle();
                item.setCrawlerPlanId(crawlerContent.getCrawlerPlanId());
                item.setTitle(title);
                item.setTitleMd5(Md5Util.encoderByMd5(title));
                item.setCreateTimestamp(now);
                saveList.add(item);
            }
        }
        if (CollectionUtils.isNotEmpty(saveList)) {
            longArticleBaseMapper.batchInsertArticleCategory(saveList);
        }
    }

    private String buildKimiPrompt(List<String> titleList) {
        StringBuilder prompt = new StringBuilder(kimiCategoryPrompt);
        prompt.append("\n");
        for (String title : titleList) {
            prompt.append(title).append("\n");
        }
        return prompt.toString();
    }

    public void articleCategoryJobRetry() {
        List<ArticleCategory> dealList = articleCategoryRepository.getByStatusAndRetryTimesLessThan(ArticleCategoryStatusEnum.FAIL.getCode(), 3);
        for (ArticleCategory articleCategory : dealList) {
            List<String> partitionTitles = Collections.singletonList(articleCategory.getTitle());
            String prompt = buildKimiPrompt(partitionTitles);
            KimiResult kimiResult = kimiApiService.requestOfficialApi(prompt, null, null);
            long now = System.currentTimeMillis();
            JSONObject obj = null;
            if (kimiResult.isSuccess()) {
                try {
                    obj = JSONObject.parseObject(kimiResult.getResponse().getChoices().get(0).getMessage().getContent());
                } catch (Exception e) {
                    log.error(kimiResult.getResponse().getChoices().get(0).getMessage().getContent());
                }
            }
            articleCategory.setKimiResult(kimiResult.getResponseStr());
            articleCategory.setUpdateTimestamp(now);
            articleCategory.setRetryTimes(articleCategory.getRetryTimes() + 1);
            if (kimiResult.isSuccess() && Objects.nonNull(obj)) {
                List<String> keys = new ArrayList<>(obj.keySet());
                String category = obj.getString(keys.get(0));
                articleCategory.setCategory(category);
                articleCategory.setStatus(ArticleCategoryStatusEnum.SUCCESS.getCode());
                articleCategory.setFailReason(null);
            } else {
                articleCategory.setStatus(ArticleCategoryStatusEnum.FAIL.getCode());
                articleCategory.setFailReason(kimiResult.getFailReason());
            }
            articleCategoryRepository.save(articleCategory);
        }
    }

    public String getPublishContentByWxSn(String wxSn) {
        Article article = articleRepository.getByWxSn(wxSn);
        PublishAccount publishAccount = publishAccountRepository.getByGhId(article.getGhId());
        if (Objects.isNull(publishAccount)) {
            return null;
        }
        long publishTimestamp = article.getPublishTimestamp() > 0 ? article.getPublishTimestamp() * 1000 : article.getUpdateTime() * 1000;
        List<PublishContent> publishContentList = aigcBaseMapper.getNearestPublishContent(publishAccount.getId(), publishTimestamp, 100);
        PublishContent publishContent = findPublishContent(publishContentList, article.getTitle(), publishTimestamp);
        if (Objects.nonNull(publishContent)) {
            return publishContent.getId();
        } else {
            publishContentList = aigcBaseMapper.getLateNearestPublishContent(publishAccount.getId(), publishTimestamp);
            publishContent = findPublishContent(publishContentList, article.getTitle(), publishTimestamp);
            if (Objects.nonNull(publishContent)) {
                return publishContent.getId();
            }
        }
        return null;
    }
}
